@@ -7,18 +7,18 @@ Do not add this file to the repository.
 
 import datetime
 
-version = '2.0'
-date = 'Fri Sep 29 22:03:34 2017'
+version = '2.1'
+date = 'Sat Feb  3 19:12:15 2018'
 
 # Was NetworkX built from a development version? If so, remember that the major
 # and minor versions reference the "target" (rather than "current") release.
 dev = False
 
 # Format: (name, major, min, revision)
-version_info = ('networkx', '2', '0', None)
+version_info = ('networkx', '2', '1', None)
 
 # Format: a 'datetime.datetime' instance
-date_info = datetime.datetime(2017, 9, 29, 22, 3, 34, 984929)
+date_info = datetime.datetime(2018, 2, 3, 19, 12, 15)
 
 # Format: (vcs, vcs_tuple)
 vcs_info = (None, (None, None))
